/*$L-	TURN OFF LISTING */

/*	Permission is hereby granted to republish,
 *	but not for profit, any or all of this program,
 *	provided that this copyright notice is included
 *
 *	Copyright 1978, Oregon Minicomputer Software, Inc.
 *		2340 SW Canyon Road
 *		Portland, Oregon 97201
 *		(503) 226-7760
 */

/*
 *	Tektronix 4010 compatible plotting routines.
 *	Adapted from the RSTS OMSI Pascal-1.1G routines.
 *	16 April 1980
 *	Andrew Luebker and Les LaCroix
 *	Carleton College
 *	Northfield, Mn. 55057
 */

CONST	CR=15B; GRAFMOD=29; BUFMAX= 127;
TYPE	BUFSIZ= 0..BUFMAX;
	BUFFER= ARRAY[BUFSIZ] OF CHAR;

VAR	PLOT: RECORD
	   HIGHX,HIGHY: CHAR;
	   XSCALE,YSCALE: REAL;
	   XOFFSET,YOFFSET: REAL;
	   END;

	PLOTBUF: BUFFER;
	BUFCNT: BUFSIZ;

PROCEDURE FLUSH;
 PROCEDURE ADEOUT(VAR NCHAR:INTEGER; VAR NARR:BUFFER); FORTRAN;
BEGIN
 	IF BUFCNT<>0 THEN ADEOUT(BUFCNT,PLOTBUF);
	BUFCNT:= 0;
END;

PROCEDURE PUTC(C: CHAR);
BEGIN
	PLOTBUF[BUFCNT]:= C;
	BUFCNT:= BUFCNT+1;
	IF BUFCNT>BUFMAX THEN FLUSH;
END;

PROCEDURE INITPLOT;
BEGIN
	PLOT.XOFFSET:= 0.0; PLOT.YOFFSET:= 0.0;
	PLOT.XSCALE:= 1.0; PLOT.YSCALE:= 1.0;
	PLOT.HIGHY:= CHR(0); PLOT.HIGHX:= CHR(0);
	BUFCNT:= 0;
END;

PROCEDURE SLEEP(TMG,TNT:INTEGER);
/*
 *	SLEEP.PAS
 *
 *	17 April 1980
 *	Tim Yaukey and Les LaCroix
 *	Carleton College
 *	Northfield, Mn. 55057
 *
 *	Routine to sleep in OMSI Pascal-1.
 *
 *	Parameters:
 *		TMG - time magnitude.  Number of time units to sleep.
 *		TNT - time unit. Value ranges fromm 1 to 4:
 *			1 - ticks (1/100 second)
 *			2 - seconds
 *			3 - minutes
 *			4 - hours
 *
 *	Depending on the value of TNT, there may be limits on the value
 *	of TMG.  Please refer to VAX-11 Programmer's Manual.
 *
 *	This routine uses local event flag #1... beware.
 */
BEGIN
/*$C
.MCALL	MRKT$S,WTSE$S
	MRKT$S	#1, <TMG+4(%6)>, <TNT+2(%6)>	; Mark time; set LEF #1
						; when done.
	WTSE$S	#1				; Wait for flag to be set
*/
END /* SLEEP */;

PROCEDURE DRAW(X,Y: REAL);
VAR	HIGHY,LOWY,HIGHX,LOWX: CHAR;
	IX,IY: INTEGER;
BEGIN
	IX:= TRUNC(X * PLOT.XSCALE + PLOT.XOFFSET);
	IY:= TRUNC(Y * PLOT.YSCALE + PLOT.YOFFSET);

	HIGHY:= CHR(40B + ((IY DIV 40B) AND 37B));
	HIGHX:= CHR(40B + ((IX DIV 40B) AND 37B));
	LOWY:= CHR(140B + (IY AND 37B));
	LOWX:= CHR(100B + (IX AND 37B));

	IF HIGHY<>PLOT.HIGHY THEN PUTC(HIGHY);
	PUTC(LOWY);					/* 4002,4010 */
/*
 *	IF (LOWY<>PLOT.LOWY) OR (HIGHX<>PLOT.HIGHX)	/* 4010 ONLY
 *	   THEN PUTC(LOWY);
 */
	IF HIGHX<>PLOT.HIGHX THEN PUTC(HIGHX);
	PUTC(LOWX);

	PLOT.HIGHY:= HIGHY;
	PLOT.HIGHX:= HIGHX;
END;

PROCEDURE MOVE(X,Y: REAL);
BEGIN
	PUTC(CHR(GRAFMOD));
	DRAW(X,Y);
END;

PROCEDURE ALPHA;
CONST	ALPHAMODE= 31;
BEGIN
	PLOT.HIGHX:= CHR(0); PLOT.HIGHY:= CHR(0);
	PUTC(CHR(ALPHAMODE));
	FLUSH;
END;

PROCEDURE ERASE;
CONST	ESC= 155; FF= 140; CAN= 152; HOME= 129;
VAR	I: INTEGER;
BEGIN
	PUTC(CHR(ESC));PUTC(CHR(FF));	/* ERASE 4010 SERIES */
	PUTC(CHR(CAN));PUTC(CHR(HOME));	/* AND 4002 SERIES */
	FOR I:=1 TO 15 DO PUTC(CHR(0));	/* 1/2 SECOND FILL */
	ALPHA;
	SLEEP(2,2);			/* 2 SECOND DELAY */
END;

PROCEDURE CROSSHAIRS(VAR CH: CHAR; VAR X,Y: REAL);
CONST	ESC= 155; SUB= 154;
VAR	XH,XL,YH,YL: CHAR;
	IX,IY: INTEGER;
BEGIN
	PUTC(CHR(ESC));PUTC(CHR(SUB));
	FLUSH;
	READLN(CH,XH,XL,YH,YL);
	IX:= (ORD(XH) AND 37B)*32+(ORD(XL) AND 37B);
	IY:= (ORD(YH) AND 37B)*32+(ORD(YL) AND 37B);
	X:= IX; Y:= IY;
END;

PROCEDURE CURLYHAIRS(VAR X,Y: REAL);
CONST	ESC= 155; SUB= 154; ENQ= 133;
VAR	XH,XL,YH,YL: CHAR;
	IX,IY: INTEGER;
BEGIN
	PUTC(CHR(ESC));PUTC(CHR(SUB));
	PUTC(CHR(ESC));PUTC(CHR(ENQ));
	FLUSH;
	READLN(XH,XL,YH,YL);

	IX:= (ORD(XH) AND 37B)*32+(ORD(XL) AND 37B);
	IY:= (ORD(YH) AND 37B)*32+(ORD(YL) AND 37B);
	X:= IX; Y:= IY;
END;

PROCEDURE DOT(X,Y: REAL);
BEGIN
	MOVE(X,Y);
	DRAW(X,Y);
END;

PROCEDURE ENDPLOT;
BEGIN
	PLOT.XOFFSET:=0; PLOT.YOFFSET:=0;
	MOVE(0,767);
	ALPHA;
END;

/*$L+	TURN ON LISTING */
