$! FORGO.COM		1 step Fortran compile, link and execute
$!
$! This procedure also deletes the object file created, and purges
$! the executable files, in an effort to conserve disk space.
$! Also allows compiler switches to be specified as parameters,
$! even if no parameters are specified (i.e., user is prompted for
$! filename, she can give qualifiers there separated by blanks
$! and/or slashes).  Also, the compiler switch "/CHECK=ALL"
$! is assumed to be in force unless overridden by another /CHECK
$! qualifier.
$! FORGO checks for multiple file specifications (not set up to 
$! do it), and for existence of file with filetype of ".FOR" if no 
$! file type specified.
$!
$check_p1:
$	if p1 .nes. "" then goto get_qual
$! no parameters given; prompt for filename, check for add'l parameters
$	inquire response "File name"
$	i=1		!parameter number being checked
$! separate response to look like passed parameters
$get_parm:
$	l=0		!offset
$	len='f$length(response)
$find_non_blank:
$	if l .eq. len then goto check_p1
$	if "''f$extract(l,1,response)'" .nes. " " then goto found_it
$	l=l+1
$	goto find_non_blank

$found_it:
$	if i .gt. 8 then goto maxparm
$	response := 'f$extract(l,len,response)	!peel off leading blanks
$	bl = 'f$locate(" ",response)		!next blank
$	p'i := 'f$extract(0,bl,response)
$	response:='f$extract(bl,len,response)	!rest of response
$	i=i+1
$	goto get_parm

$! now put the compiler switches together from the parameters given:
$get_qual:
$	file := 'p1
$	if 'f$locate(",",file) .ne. 'f$length(file) then goto file_error
$	switch := ""		!compiler switches
$	l='f$locate("/",file)		!check for switches in file spec
$	len='f$length(file)-1
$	switch := 'switch''f$extract(l,len,file)'
$	file := 'f$extract(0,l,file)
$	i=2
$switchloop:
$	if p'i .eqs. "" then goto check_switch
$	sw := "'p''i'"		!sw = 'Pi, so next line will be right
$	sw := 'sw		!now sw is the actual switch
$	if "''f$extract(0,1,sw)'" .nes. "/" then goto switch_error
$	switch := 'switch''sw'
$	i=i+1
$	if i .le. 8 then goto switchloop	!max parameters = 8


$check_switch:
$	linkswitch := ""		!switches for link
$	if switch .eqs. "" then goto compile
$	len = 'f$length(switch)
$	if 'f$locate("/OB",switch) .ne. len then goto switch_error2
$	if 'f$locate("/NOOB",switch) .ne. len then goto switch_error3
$
$!If "/DEBUG" specified, then set link switch to "/DEBUG" also:
$	if 'f$locate("/DE",switch) .ne. len then linkswitch := "/DEBUG"
$
$! Now compile
$compile:
$!   first check for most common error among novices...that is,
$!   they say FORGO FILENAME and file type is null rather than .FOR.
$!   (note: will not work for compound source, or if a directory and
$!   subdirectory specified; however, in such case, user is 
$!   probably not a novice...)
$	len='f$length(file)
$	if 'f$locate("+",file) .ne. len then goto compile2
$	if 'f$locate(".",file) .ne. len then goto compile2
$	open/error=not_found file 'file.for
$	close file		!file type .for exists
$	goto compile2
$not_found:
$	write sys$output "FORGO warning -- file """,file,".FOR"" not found"
$	write sys$output "file """,file,"."" assumed"
$	file:='file'.
$
$compile2:
$!  strip file type off first file specification, for deleting .obj, etc.
$	infile:='file			!save full input file name(s)
$	l='f$locate("+",file)
$	file:='f$extract(0,l,file)	!first one only
$	l='f$locate(":",file)+1		!is there a disk or logical name?
$	if l-1 .ne. 'f$length(file) then file:='f$extract(l,80,file)
$	l='f$locate("]",file)+1		!check for file type
$	if l-1 .ne. 'f$length(file) then file:='f$extract(l,15,file)
$	l:='f$locate(".",file)		!file type spec.?
$	file:='f$extract(0,l,file)	!strip off file type, if any
$
$	on control_y then goto del_obj
$	on error then goto compile_error
$	switch := "/CHECK=ALL"'switch	!add default switch
$	fortran'switch  'infile		!compile
$	if $severity .eq. 0 then goto warning
$	if .not.$severity then goto compile_error	!even => failure
$
$compile3:
$!  Check if a ".EXE" file exists already, and set exe_switch accordingly
$	exe_switch := "N"			!to test whether to purge
$	open/error=link file 'file.exe	!is there an .exe already?
$	exe_switch := "Y"			!yes
$	close file
$
$! Now link the file, with the /DEBUG switch, if specified for compile
$link:
$	on control_y then goto cleanup
$	on warning then goto link_error
$	link'linkswitch 'file			!link
$	if .not. $severity then goto link_error	!even => failure
$	on error then goto cleanup
$	assign/user tt: sys$input
$	run 'file
$
$cleanup:
$	set nocontrol_y
$	if exe_switch then purge 'file.exe	!purge if already > 1.
$del_obj:
$	set nocontrol_y
$	delete 'file.obj;		!get rid of object file
$
$finish:
$	set control_y
$	exit

$! *****************************************************************
$!
$! WARNINGS:
$
$warning:
$	write sys$output "warnings encountered during compile"
$	inquire go "do you want to continue with link and run?  "
$	if go then goto compile3
$	goto del_obj			!no, delete object file
$
$! ERRORS:

$maxparm:
$	write sys$output "FORGO error: too many parameters (max=8)"
$	exit

$file_error:
$	write sys$output "FORGO error: multiple files not allowed"
$	write sys$output "use '+' to concatenate multiple files"
$	exit

$switch_error:
$	write sys$output -
	"FORGO error: '",sw,"' is illegal; switches must begin with '/'"
$	exit

$switch_error2:
$	write sys$output "FORGO error: '/OBJECT' switch not allowed"
$	exit

$switch_error3:
$	write sys$output "FORGO error: '/NOOBJECT' switch not allowed"
$	exit

$compile_error:
$	set nocontrol_y
$!  compile error...was an object module created?
$	open/error=compile_error2 file 'file.obj
$	close file
$	delete 'file.obj.		!yes, delete it...
$compile_error2:
$	write sys$output "errors encountered in compile...link not attempted."
$	goto finish

$link_error:
$	set nocontrol_y
$	write sys$output -
		"errors encountered during link...run not attempted"
$	goto cleanup
